/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * This code developed by The Department of Biomedical and Health Informatics at 
 * at the University of Washington (http://www.bhi.washington.edu)
 *
 */
 
package edu.washington.bhi.protegex.storage.xml;

//import util.Arguments;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;

/* Writes out XML from Protege
 */
public class ProtegeXMLCreator extends XMLCreator{
	
	//do i really want to do this?  pass kb as an almost global var rather
	//than pass other kbs?  Probably ..... FIX
	protected KnowledgeBase mainkb;
	
    
    static final String XML_SCHEMA_NAME = "protegekb";

	public ProtegeXMLCreator(KnowledgeBase kb)
	{
	  super();
      mainkb = kb;
      initializeMain(XML_SCHEMA_NAME);
      addHeaders();
      addMaintoDoc();
	}
	
	
	private void addClasses() {
		//root is the root for the 2nd layer
		//item is the child of root
		Element root2, root,item, item2, item3;
		
		//add the top level <classes> tag
		root2 = addRootElement(main,"classes");

		int element_count = 0;
		
		Collection classes = mainkb.getClses();
//		classes = sort(classes);
		Iterator i = classes.iterator();
		
		while (i.hasNext()) {
		    Cls cls = (Cls) i.next();
		    if (!cls.isSystem() && !cls.isIncluded()){
			    //add <class>
				root = addRootElement(root2,"class");

				element_count++;
			    
			    //add own-slots
			    //add own-slot start
			    item = addChildElement(root, "own-slots", "");
			    	            				
	            //add type
	            //fix for metaclasses
	            //find instances of this class and figure it out from there
	            Cls metaCls = cls.getDirectType();
				
				//add own slots from meta-class and make them own-slots for this class
				Collection ownslots = cls.getOwnSlots();
				
			
				if (ownslots != null) {
					Iterator j = ownslots.iterator();
					while(j.hasNext()){
					    Slot slot = (Slot)j.next();
				        //value is collection of ownslots
					    Collection slotColl = cls.getOwnSlotValues(slot);
					    if(slotColl == null || slotColl.isEmpty())
						continue;		
						String slotName = slot.getName(); 
						
						//if slot is name slot then stick it at the front
						if (slotName.equals(Model.Slot.NAME)) { 
							addOwnslot(item, slotName, slotColl, true);
						}
						else if (slotName.equals(Model.Slot.DIRECT_SUBCLASSES))
							continue;
						else if (slotName.equals(Model.Slot.DIRECT_INSTANCES))
							continue;
						else {
							addOwnslot(item, slotName, slotColl, false);
						}
					}
				}						
	     		//end own-slots	
								
						
				item2 = addChildElement(root, "slot-overrides", "");
				boolean existsOverrides = false;
				boolean hasMaxCard = false;
				Collection values;

				//loop through list of facets and see if it's the one
				Iterator j = cls.getTemplateSlots().iterator();
				while (j.hasNext()) {
					Slot slot = (Slot)j.next();
					String name = slot.getName();
					if (cls.hasDirectlyOverriddenTemplateSlot(slot)) {
				//	if (cls.hasOverriddenTemplateSlot(slot)) {
						hasMaxCard = false;
						existsOverrides = true;
						item3 = addChildElement(item2, "slot-override", "");
						addAttribute(item3, "slot-name", slot.getName());
						Collection facets = cls.getTemplateFacets(slot);
						Iterator k = facets.iterator();
						while (k.hasNext()) {
							Facet facet = (Facet)k.next();
							String fname = facet.getName();
							if (cls.hasDirectlyOverriddenTemplateFacet(slot, facet)) {
								String facetName = facet.getName();
							    values = cls.getDirectTemplateFacetValues(slot, facet);
							    if(values == null || values.isEmpty())
								continue;
								
								if (facetName.equals(Model.Facet.MAXIMUM_CARDINALITY)) {
									hasMaxCard = true;
								}
								
								addOverride(item3, facetName, values);
							}
						}
												
						//check maxcardinality infinite manually
						if (!hasMaxCard && cls.getTemplateSlotAllowsMultipleValues(slot)) {
							values = new HashSet();
							Integer staticMaxCard = new Integer(KnowledgeBase.MAXIMUM_CARDINALITY_UNBOUNDED);
							values.add(staticMaxCard);
							addOverride(item3, Model.Facet.MAXIMUM_CARDINALITY, values);
						}						
					}
				}
				
				if (!existsOverrides)
					deleteNode(root, item2);				 
		    }
		}
		
	    if (element_count > 0) {
			//add to the Top Element
			addElementtoMain(root2);
		}
	}                                                                                                                                  
	
	
	private void addSlots() {
		
		Element root,item, root2;

	    root2 = addRootElement(main,"slots");

		int element_count = 0;
		
//		Iterator i = mainkb.getSlots().iterator();
		Collection slots = mainkb.getSlots();
		slots = sort(slots);
		Iterator i = slots.iterator();
		while (i.hasNext()) {
		    Slot slot = (Slot) i.next();
		    
		    if (!slot.isSystem() && !slot.isIncluded()){
		    	
		    	//add top level <slot> declaration
			    root = addRootElement(root2,"slot");
			    
			    element_count++;
			    
				//add own slots from meta-class and make them own-slots for this class
				Collection ownslots = slot.getOwnSlots();
				boolean hasMaxCard = false;
				Collection slotColl;
				
				if (ownslots != null) {
					Iterator j = ownslots.iterator();
					while(j.hasNext()){
					    Slot slot2 = (Slot)j.next();
				        //value is collection of ownslots
					    slotColl = slot.getOwnSlotValues(slot2);
					    if(slotColl == null || slotColl.isEmpty())
						continue;		
						String slotName = slot2.getName(); 
						
						//if slot is name slot then stick it at the front
						if (slotName.equals(Model.Slot.NAME)) { 
							addOwnslot(root, slotName, slotColl, true);
						}
						else if (slotName.equals(Model.Slot.DIRECT_SUBCLASSES))
							continue;
						else if (slotName.equals(Model.Slot.DIRECT_INSTANCES))
							continue;
						else if (slotName.equals(Model.Slot.MAXIMUM_CARDINALITY)) {
							hasMaxCard = true;
							addOwnslot(root, slotName, slotColl, false);
						}
						else {
							addOwnslot(root, slotName, slotColl, false);
						}
					}
				}
				
				//check maxcardinality infinite manually
				if (!hasMaxCard && slot.getOwnSlotAllowsMultipleValues(slot)) {
					slotColl = new HashSet();
					Integer staticMaxCard = new Integer(KnowledgeBase.MAXIMUM_CARDINALITY_UNBOUNDED);
					slotColl.add(staticMaxCard);
					addOwnslot(root, Model.Slot.MAXIMUM_CARDINALITY, slotColl, false);
				}							
	     		//end own-slots		
	     	}
		  }
		  
		if (element_count > 0) {
			//add to the Top Element
			 addElementtoMain(root2);
		}
	}
	
	private void addFacets() {
		Element root,item, root2;
		
	    root2 = addRootElement(main,"facets");
	   	int element_count = 0;
		
		Iterator i = mainkb.getFacets().iterator();
		while (i.hasNext()) {
		    Facet facet = (Facet) i.next();
		    
		    if (!facet.isSystem() && !facet.isIncluded()){
		    	
		    	//add top level <facet> declaration
			    root = addRootElement(root2,"facet");
				element_count++;
		        
			    //add name Element
			    addChildElement(root, "name", facet.getName());
	            
	            //associated-slot (slot-name)
	            Slot slot = facet.getAssociatedSlot();
			  	if(facet != null) {
					addChildElement(root, "associated-slot", slot.getName());
	         	}
				//documentation (maybe not?)
	    
	         }
		  }

	    if (element_count > 0) {       
			//add to the Top Element
	    	addElementtoMain(root2);
	    }
	}
	
	private void addInstances() {
		Collection instances = mainkb.getInstances();
		Element root;
		//sort the instances
	//	instances = sortInstances(instances);
	
		int element_count = 0;
				
		root = addRootElement(main,"instances");
		
		Iterator i = instances.iterator();
			while (i.hasNext()) {
			    Instance instance = (Instance) i.next();
			    if (instance instanceof SimpleInstance && !instance.isSystem()) {
//				if (!instance.isSystem()) {
				// save instance
				saveInstance(instance, mainkb, root);
				element_count++;
			    }
		}
		if (element_count > 0) {
			addElementtoMain(root);
		}
	}
	
	private void saveInstance(Instance instance, KnowledgeBase kb, Element root2){
		Element root,item;
		Cls cls;

		cls = instance.getDirectType();
	
		if(instance.getDirectType() == null){
		    return;
		}
	
		String className = cls.getName();
	
		//create instance element to tack on to list
		root = addRootElement(root2,"instance");
	
		//add id
		addChildElement(root, "id", instance.getName());
		
		//add type (which class)
		addChildElement(root, "type", className);
		item = addChildElement(root, "own-slots", "");
		
		//add template slots and values
		//each template slot looks like this:
		//<own-slot><slot/><value/></own-slot>
		
		Collection template_slots = cls.getTemplateSlots();
	
		if (template_slots != null) {
			Iterator i = template_slots.iterator();
			while(i.hasNext()){
			    Slot slot = (Slot)i.next();
			    
			    //value is collection of ownslots
			    Collection values = instance.getOwnSlotValues(slot);
			    if(values == null || values.isEmpty())
				continue;		
				String slotName = slot.getName();  
				addOwnslot(item, slotName, values, false);
			}					
		}

    } //end saveinstances  
    
    private Element addOverride(Element root, String facetname, Collection values) {
		//follow the pattern: <override facet-name/><value/>
		//Collection values is a collection of strings
		
		Element item, child;
		
		//create the <override top>
		item = addChildtoFront(root, "override", "");

		addAttribute(item, "facet-name", facetname);		
		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();
				ValueType type = getObjectValueType(obj);
				String entry = convertObjecttoString(obj, type);				
				String stringType = type.toString();
				addEntry(item, stringType, entry);
			}
		}
		
		return item;
	}
  
    
    private Element addOwnslot(Element root, String slotname, Collection values, boolean addToFront) {
		//follow the pattern: <own-slot slot-name><value><type>
		//Collection values is a collection of strings
		
		Element item, child, child2;
		
		//create the <own-slot top>
		if (addToFront) {
			item = addChildtoFront(root, "own-slot", "");
		}
		else {
			item = addChildElement(root, "own-slot", "");
		}

		addAttribute(item, "slot-name", slotname);		
		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();
				ValueType type = getObjectValueType(obj);
				String entry = convertObjecttoString(obj, type);				
				String stringType = type.toString();
				addEntry(item, stringType, entry);
			}
		}
		
		return item;
	}

	private Element addEntry(Element root, String type, String value) {
		Element child = addChildElement(root, "entry", "");
		addAttribute(child, "type", type);
		Element child2 = addChildElement(child, "value", value);
		
		return child;
	}

    private Element addFacet(Element root, String facetname, Collection values, boolean addToFront) {
		//follow the pattern: <own-slot slot-name><value><type>
		//Collection values is a collection of strings
		
		Element item, child;
		
		//create the <own-slot top>
		if (addToFront) {
			item = addChildtoFront(root, "facet", "");
		}
		else {
			item = addChildElement(root, "facet", "");
		}

		addAttribute(item, "facet-name", facetname);		
		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();
				ValueType type = getObjectValueType(obj);
				String entry = convertObjecttoString(obj, type);				
				String stringType = type.toString();
				child = addChildElement(item, "value", entry);
				addAttribute(child, "type", stringType);
			}
		}
		
		return item;
	}

    
    protected String getTypeName(ValueType type){
		if(type == null)
		    return "Unknown";
		else
			return type.toString();
    }

    protected void addHeaders() {
    	addAttribute(main, "xmlns:xsi", "http://www.w3.org/2001/XMLSchema-instance");
		addAttribute(main, "xsi:noNamespaceSchemaLocation", "http://protege.stanford.edu/plugins/xmlbackend/protege_xml_backend.xsd");
// 		addAttribute(main, "xsi:noNamespaceSchemaLocation", "protege_xml_backend.xsd");

    }

	public Node addElementsToDocument() {
		addSlots();
		addClasses();
//		addFacets();
		addInstances();
		return main;
	}
	
	private String returnObjectName(Object o) {
        String s;
          if (o instanceof Cls || o instanceof Slot || o instanceof Facet) {
            s = ((Instance) o).getName();
          } else if (o instanceof Instance) {
            s = ((Instance) o).getName();
          } else if (o instanceof Boolean) {
            s = ((Boolean) o).booleanValue() ? "TRUE" : "FALSE";
          } else {
            s = o.toString();
          }
        return s;
    }
    
    private Collection convertColltoString(Collection coll) {
		Collection newColl = new HashSet();
		
		Iterator i = coll.iterator() ;
		while(i.hasNext()){
	    	String value = i.next().toString();
	    	newColl.add(value);
	    }
		return newColl;
	}
	
}