/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * This code developed by The Department of Biomedical and Health Informatics at 
 * at the University of Washington (http://www.bhi.washington.edu)
 *
 */

package edu.washington.bhi.protegex.storage.xml;

import java.io.*;
import java.util.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

import org.w3c.dom.*;


/**
 * XML schema backend for Protege 2000
 * 
 * Anton An
 * June 19, 2001
 * Ping-Ping Lin
 * October 16, 2002
 */

public class XMLBackend implements KnowledgeBaseFactory {
    
    static final String XML_FILE_NAME_PROPERTY = "xml_file_name";

	private ProtegeXMLCreator pxmlc;
	private ProtegeXMLImporter pxmli;
	private XMLPrinter XMLprinter;	
	private Node XMLtree;
	private String XMLfilename;

    /**
     * Create a new knowledge base.  This implementation just uses 
     * DefaultKnowledgeBase.  
     */
    public KnowledgeBase createKnowledgeBase(Collection errors) {
        KnowledgeBase kb = new DefaultKnowledgeBase(this);
        return kb;
    }

    public void includeKnowledgeBase(KnowledgeBase kb, PropertyList sources, 
				     Collection errors) {
       	load(kb, sources, errors, true);
    }

    public void loadKnowledgeBase(KnowledgeBase kb, PropertyList sources, 
				  Collection errors) {
		load(kb, sources, errors, false);
    }
    

    private void load(KnowledgeBase kb, PropertyList sources, 
		      Collection errors, boolean isIncluded) {
		      	
		String tempname = kb.getName();
		
		//get file name from pprj
		XMLfilename = getXMLFileName(sources);
		
		//get the path
	 	Project project = kb.getProject();
        String projectDir = FileUtilities.getDirectory(project.getProjectFilePath()) + "/";
        			
		pxmli = new ProtegeXMLImporter(kb, isIncluded);

		System.out.println("Loading XML from " + projectDir + XMLfilename + "...");	
		XMLtree = pxmli.loadfromFile(projectDir + XMLfilename);
		
		System.out.println("Parsing XML...");	
		pxmli.parseDocument(XMLtree);		
    }

    public void saveKnowledgeBase(KnowledgeBase kb, PropertyList sources, 
				  Collection errors) {

       //initialize writer with KB and Writer object? 
	 	XMLfilename = getXMLFileName(sources);
	 	
	 	//get the path
	 	Project project = kb.getProject();
        String projectDir = FileUtilities.getDirectory(project.getProjectFilePath()) + "/";

  	  	pxmlc = new ProtegeXMLCreator(kb);
    	   
       //load stuff into xerces node tree
       System.out.println("Creating XML...");	
		XMLtree = pxmlc.addElementsToDocument();
       
       //print stuff
       System.out.println("Writing XML to file...");	
  		XMLprinter = new XMLPrinter();
   		XMLprinter.printXML(XMLtree, projectDir + XMLfilename);         
    }


    /** Create a source editor */
    public KnowledgeBaseSourcesEditor createKnowledgeBaseSourcesEditor(
          String projectName, PropertyList sources) {
      return new XMLBackendSourcesEditor(projectName, sources);
    }

    public boolean isComplete(PropertyList sources) {
	return true;
    }
    
    /**
     * returns the description of this backend for display to users
     */
    public String getDescription() {
        return "XML Backend";
    }

    /**
     * returns the path to any project file containing customized forms which are needed by this backend.  The
     * file must be in the jar and the path must be relative to this class file location in the jar.  Return
     * null if no customized forms are needed.
     */
    public String getProjectFilePath() {
        return null;
    }

    public String getLabel() {
        return "XML Backend";
    }

    /**
     * a helper method to extract the name of the file for storing classes from the property list
     */
    public static String getXMLFileName(PropertyList sources) {
        return sources.getString(XML_FILE_NAME_PROPERTY);
    }

    /**
     * helper method to stuff the class and instances file names into the sources property list
     */
    public static void setSourceFiles(PropertyList sources, String XMLFileName) {
        sources.setString(XML_FILE_NAME_PROPERTY, pathToName(XMLFileName));
    }


    private static String pathToName(String path) {
        return new File(path).getName();
    }

    /**
     * method to make this class easy to debug
     */
    public static void main(String[] args) {
        edu.stanford.smi.protege.Application.main(args);
    }
}