/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * This code developed by The Department of Biomedical and Health Informatics at 
 * at the University of Washington (http://www.bhi.washington.edu)
 *
 */

package edu.washington.bhi.protegex.storage.xml;

import java.awt.*;
import java.io.*;
import javax.swing.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;

/**
 *  Editor for fields specific to this backend.
 *
 * @author    Ray Fergerson <fergerson@smi.stanford.edu>
 */
public class XMLBackendSourcesEditor extends KnowledgeBaseSourcesEditor {
    private FileField XMLField;

    /**
     * create widgets to acquire everything necessary for a save to be performed successfully.  In
     * this example what is needed is two relative file names, one to store the classes and one
     * to store the instances.
     */
    public XMLBackendSourcesEditor(String projectPath, PropertyList sources) {
        super(projectPath, sources);
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 1, 10, 10));
        panel.add(createXMLField());
        add(panel);
    }

    /**
     * construct a file name from the base project name and an extension
     */
    private String constructName(String projectPath, String extension) {
        String name = null;
        if (projectPath != null) {
            int index = projectPath.lastIndexOf(".");
            if (index != -1) {
                name = projectPath.substring(0, index) + extension;
            }
        }
        return name;
    }

    private JComponent createXMLField() {
        String name = XMLBackend.getXMLFileName(getSources());
        if (name == null) {
            name = constructName(getProjectPath(), ".xml");
        }
        XMLField = new FileField("XML FileName", name, ".xml", "Ontology");
        return XMLField;
    }

    private boolean hasValidValue(FileField field) {
        boolean hasValidValue;
        String value = field.getPath();
        if (value == null || value.length() == 0) {
            hasValidValue = false;
        } else {
            hasValidValue = true;
        }
        return hasValidValue;
    }

    /**
     * the system calls this method when the user changes the project name.  We
     * should update any files whose names may depend on the project name.
     */
    public void onProjectPathChange(String oldPath, String newPath) {
        if (newPath != null) {
            updatePath(XMLField, ".xml");
        }
    }

    /**
     * the system calls this method when the user presses the OK button and the contents
     * have been declared to be valid.  This method should stuff any
     * configuration information into the sources property list.
     */
    public void saveContents() {
        String XMLFileName = XMLField.getPath();
        XMLBackend.setSourceFiles(getSources(), XMLFileName);
    }

    private void updatePath(FileField field, String ext) {
        File projectFile = new File(getProjectPath());
        String projectName = projectFile.getName();
        String projectDir = projectFile.getParent();
        int index = projectName.lastIndexOf(".");
        if (index != -1) {
            projectName = projectName.substring(0, index);
        }
        String newPath = projectDir + "\\" + projectName + ext;
        field.setPath(newPath);
    }

    /**
     * called by the system when the user presses the OK button.  return true if the contents
     * of the fields are valid.  Otherwise pop up a dialog with a description of the problem and return false.
     */
    public boolean validateContents() {
        boolean isComplete = hasValidValue(XMLField);
        if (!isComplete) {
            // TODO: pop up a dialog to warn the user.
            isComplete = false;
        }
        return isComplete;
    }
}
